/* 
 * glSpace - 3D motor to render three-dimentional objects
 * Copyright (C) 2001 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/* Compile: gcc -ansi -O2 -o glSpace glSpace.c camera.c stars.c
   -L/usr/X11R6/lib -lglut -lGLU -lGL -lXmu -lXext -lXi -lX11 -lm */

#ifdef WIN32
#include "windows.h"
#endif

#include <stdio.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

#include "camera.h" /* by David Farell */
#include "stars.h"

/*
 * Include the .h file with the display list here
 * Since each file has unique function names, you
 * can include as many of the display lists here
 * as you want. Be sure the corresponding .gl files
 * are in the current directory.
 */
#include "hubble.h"
#include "tfighter.h"

/* Globals for current state and the 3D data */
static int model[2];
static int roll=1;

static int move_left=0;
static int move_right=0;
static int move_up=0;
static int move_down=0;
static int move_forward=0;
static int move_back=0;

static int pitch_up=0;
static int pitch_down=0;
static int yaw_right=0;
static int yaw_left=0;
static int roll_left=0;
static int roll_right=0;


static void key (unsigned char k, int x, int y) {
  switch(k) {
  case 27:
    exit(0);
    break;
  case 'q':
    move_forward = 1;
    break;
  case 'w':
    move_back = 1;
    break;
  case 'y':
    roll = 1 - roll;
    break;
  case 'r':
    CameraReset();
    break;
  }
}
static void keyup (unsigned char k, int x, int y) {
  switch(k) {
  case 'q':
    move_forward = 0;
    break;
  case 'w':
    move_back = 0;
    break;
  }
}

static void special (int k, int x, int y) {
  switch (k) {
  case GLUT_KEY_UP:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_up = 1;
    else
      pitch_down = 1;
    break;
  case GLUT_KEY_DOWN:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_down = 1;
    else
      pitch_up = 1;
    break;
  case GLUT_KEY_LEFT:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_left = 1;
    else
      if(roll)
	roll_left = 1;
      else
	yaw_right = 1;
    break;
  case GLUT_KEY_RIGHT:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_right = 1;
    else
      if(roll)
	roll_right = 1;
      else
	yaw_left = 1;
    break;
  }
}

static void specialup (int k, int x, int y) {
  switch (k) {
  case GLUT_KEY_UP:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_up = 0;
    else
      pitch_down = 0;
    break;
  case GLUT_KEY_DOWN:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_down = 0;
    else
      pitch_up = 0;
    break;
  case GLUT_KEY_LEFT:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_left = 0;
    else
      if(roll)
	roll_left = 0;
      else
	yaw_right = 0;
    break;
  case GLUT_KEY_RIGHT:
    if(glutGetModifiers() & GLUT_ACTIVE_SHIFT)
      move_right = 0;
    else
      if(roll)
	roll_right = 0;
      else
	yaw_left = 0;
    break;
  }
}

static void draw() {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  /* Move player */
  if(move_left) moveLeft();
  else if(move_right) moveRight();
  if(move_up) moveUp();
  else if(move_down) moveDown();
  if(move_forward) moveForward();
  else if(move_back) moveBack();
  if(pitch_up) pitchUp();
  else if(pitch_down) pitchDown();
  if(yaw_right) yawRight();
  else if(yaw_left) yawLeft();;
  if(roll_left) rollLeft();
  else if(roll_right) rollRight();;

  glPushMatrix();
  DrawStars();

  glPushMatrix();
  glCallList(model[1]);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(100.0, 0.0, -600.0);
  glRotatef(60.0, 0.0, 1.0, 1.0);
  glCallList(model[0]);
  glPopMatrix();

  glPushMatrix();
  glTranslatef(0.0, 500.0, 0.0);
  glRotatef(40.0, 1.0, 0.0, 1.0);
  glCallList(model[0]);
  glPopMatrix();

  glPopMatrix();

  glutSwapBuffers();
}

static void idle() {
  glutPostRedisplay();
}

/* 
 * Basic OpenGL initialization
 * It turns on lighting, back face culling, depth
 * buffer, etc. 
 */
void init() {
  GLfloat global_ambient[4]={0.2, 0.2, 0.2, 1.0};
  GLfloat light0pos[4]=     {0.0, 0.0, 0.0, 1.0};
  GLfloat light0ambient[4]= {0.2, 0.2, 0.2, 1.0};
  GLfloat light0diffuse[4]= {0.6, 0.6, 0.6, 1.0};
  GLfloat light0specular[4]={0.8, 0.8, 0.8, 1.0};

  glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
  glutInitWindowSize(640, 360);
  if (glutCreateWindow("_ glSpace _") == GL_FALSE)
    exit(-1);

  glViewport(0, 60, 640, 360);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60, 16./9., 5,  2000);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
  glFrontFace(GL_CCW);

  glDisable(GL_DITHER);
  glShadeModel(GL_SMOOTH);

  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, global_ambient);
  glLightfv(GL_LIGHT0, GL_POSITION, light0pos);
  glLightfv(GL_LIGHT0, GL_AMBIENT, light0ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, light0diffuse);
  glLightfv(GL_LIGHT0, GL_SPECULAR, light0specular);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

  glMateriali(GL_FRONT, GL_SHININESS, 128);
  glClearColor(0.0, 0.0, 0.0, 1.0); 
}

int main(int argc, char **argv) {
  init();

  /* 
   * Here's the function call to load and compile the display list
   * data. It returns an int as the display list ID number.
   *
   */
  model[0] = GL3DS_initialize_hubble();
  if (model[0] == -1) {
    printf("Error initializing display list\n");
    exit (-1);
  }
  model[1] = GL3DS_initialize_tfighter();
  if (model[1] == -1) {
    printf("Error initializing display list\n");
    exit (-1);
  }

  InitStars();
  CameraReset();

  glutIgnoreKeyRepeat(1);

  glutKeyboardFunc(key);
  glutKeyboardUpFunc(keyup);

  glutSpecialFunc(special);
  glutSpecialUpFunc(specialup);

  glutDisplayFunc(draw);
  glutIdleFunc(idle);

  glutMainLoop();

  return(0);
}






